// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Airbridge/Container/AirbridgeContainer.h"
#include "Interfaces/AirbridgeCore.h"

#define SUPPORTED_PLATFORM (PLATFORM_ANDROID || PLATFORM_IOS)

/**
 * Airbridge Unreal SDK
 */
class AIRBRIDGEUNREAL_API FAirbridge
{
public:

#pragma region Core

	/**
	 * Enables the SDK.
	 */
	static void EnableSDK();
	
	/**
	 * Disables the SDK.
	 */
	static void DisableSDK();
	
	/**
	 * Checks whether the SDK is currently enabled.
	 * @returns `true` if the SDK is enabled, `false` otherwise.
	 */
	static bool IsSDKEnabled();

	// static void StartInAppPurchaseTracking();
	// static void StopInAppPurchaseTracking();
	// static bool IsInAppPurchaseTrackingEnabled();
    
#pragma endregion
	
#pragma region Privacy

	/**
	 * Start collecting and transferring events.
	 */
	static void StartTracking();

	/**
	 * Stop collecting and transferring events.
	 */
	static void StopTracking();
	
	/**
	 * Checks whether the SDK is currently enabled for tracking.
	 * @returns `true` if the SDK is enabled for tracking, `false` otherwise.
	 */
	static bool IsTrackingEnabled();
    
#pragma endregion

#pragma region Deeplink

	/**
	 * Handles deeplink and deferred-deeplink.
	 * @param OnDeeplinkReceived A callback listener that receives a delivered deeplink URL.
	 * <example>
	 * **Example usage**
	 * @code
	 * FAirbridge::SetOnDeeplinkReceived([](const FString& Url)
	 * {
	 *     // Process deeplink data
	 * });
	 * @endcode
	 * </example>
	 */
	static void SetOnDeeplinkReceived(const AirbridgeStringParamFunction& OnDeeplinkReceived);
    
#pragma endregion

#pragma region Data Collection

	/**
	 * Sets the user ID.
	 * @param Id The user ID.
	 */
	static void SetUserID(const FString& Id);

	/**
	 * Clears the user ID. 
	 */
	static void ClearUserID();

	/**
	 * Sets the user email.
	 * @param Email The user email.
	 */
	static void SetUserEmail(const FString& Email);

	/**
	 * Clears the user email.
	 */
	static void ClearUserEmail();

	/**
	 * Sets the user phone number.
	 * @param Phone The user phone number.
	 */
	static void SetUserPhone(const FString& Phone);

	/**
	 * Clears the user phone number.
	 */
	static void ClearUserPhone();

	/**
	 * Sets the key, value pair to the user attribute.
	 * @param Key The key that uniquely identifies the user attribute.
	 * @param Value The value to set for the user attribute.
	 */
	static void SetUserAttribute(const FString& Key, const int32 Value);

	/**
	 * Sets the key, value pair to the user attribute.
	 * @param Key The key that uniquely identifies the user attribute.
	 * @param Value The value to set for the user attribute.
	 */
	static void SetUserAttribute(const FString& Key, const int64 Value);

	/**
	 * Sets the key, value pair to the user attribute.
	 * @param Key The key that uniquely identifies the user attribute.
	 * @param Value The value to set for the user attribute.
	 */
	static void SetUserAttribute(const FString& Key, const float Value);

	/**
	 * Sets the key, value pair to the user attribute.
	 * @param Key The key that uniquely identifies the user attribute.
	 * @param Value The value to set for the user attribute.
	 */
	static void SetUserAttribute(const FString& Key, const double Value);

	/**
	 * Sets the key, value pair to the user attribute.
	 * @param Key The key that uniquely identifies the user attribute.
	 * @param Value The value to set for the user attribute.
	 */
	static void SetUserAttribute(const FString& Key, const bool Value);

	/**
	 * Sets the key, value pair to the user attribute.
	 * @param Key The key that uniquely identifies the user attribute.
	 * @param Value The value to set for the user attribute.
	 */
	static void SetUserAttribute(const FString& Key, const FString& Value);

	/**
	 * Removes the user attribute with the given key.
	 * @param Key The key that uniquely identifies the user attribute.
	 */
	static void RemoveUserAttribute(const FString& Key);

	/**
	 * Clears all user attributes.
	 */
	static void ClearUserAttributes();

	/**
	 * Sets the key, value pair to the user alias.
	 * @param Key The key that uniquely identifies the user alias.
	 * @param Value The value to set for the user alias.
	 */
	static void SetUserAlias(const FString& Key, const FString& Value);

	/**
	 * Removes the user alias with the given key.
	 * @param Key The key that uniquely identifies the user alias.
	 */
	static void RemoveUserAlias(const FString& Key);

	/**
	 * Clears all user aliases.
	 */
	static void ClearUserAlias();

	/**
	 * Clears all user information.
	 */
	static void ClearUser();

	/**
	 * Sets the key, value pair to the device alias.
	 * @param Key The key that uniquely identifies the device alias.
	 * @param Value The value to set for the device alias.
	 */
	static void SetDeviceAlias(const FString& Key, const FString& Value);

	/**
	 * Removes the device alias with the given key.
	 * @param Key The key that uniquely identifies the device alias.
	 */
	static void RemoveDeviceAlias(const FString& Key);

	/**
	 * Clears all device aliases.
	 */
	static void ClearDeviceAlias();

	/**
	 * Registers a push notification token to track app uninstalls.
	 * @param Token The push notification token, typically obtained from the device’s notification service provider,
	 * such as APNs for Apple devices or FCM registration token for Android devices.
	 */
	static void RegisterPushToken(const FString& Token);

#pragma endregion 

#pragma region Event

	/**
	 * Track events.
	 * @param Category The category of the event.
	 * @param SemanticAttributes The semantic attributes of the event.
	 * @param CustomAttributes The custom attributes of the event.
	 */
	static void TrackEvent(
		const FString& Category,
		const UAirbridgeMap* SemanticAttributes = nullptr,
		const UAirbridgeMap* CustomAttributes = nullptr
	);

#pragma endregion

#pragma region Placement

	/**
	 * Queries the tracking link to the server to get the deeplink information and then moves according to that information if a deeplink is set up.
	 * It also adds a click event for the tracking link.
	 * @param TrackingLink The URL to be queried for deeplink information and tracked for clicks.
	 * @param OnSuccess An optional callback listener to be invoked if the click action is successful.
	 * @param OnFailure An optional callback listener to be invoked if an error occurs during the click action.
	 * <example>
	 * **Example usage**
	 * @code
	 * FAirbridge::Click(TEXT("https://abr.ge/~~~"), []()
	 * {
	 *     // Handle on success
	 * }, [](const FString& ErrorMessage)
	 * {
	 *     // Handle on failure
	 * });
	 * @endcode
	 * </example>
	 */
	static void Click(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction& OnSuccess = nullptr,
		const AirbridgeStringParamFunction& OnFailure = nullptr
		);

	/**
	 * Adds an impression event for the tracking link.
	 * @param TrackingLink The URL to be tracked for impressions.
	 * @param OnSuccess An optional callback listener to be invoked if the impression action is successful.
	 * @param OnFailure An optional callback listener to be invoked if an error occurs during the impression action.
	 * <example>
	 * **Example usage**
	 * @code
	 * FAirbridge::Impression(TEXT("https://abr.ge/~~~"), []()
	 * {
	 *     // Handle on success
	 * }, [](const FString& ErrorMessage)
	 * {
	 *     // Handle on failure
	 * });
	 * @endcode
	 * </example>
	 */
	static void Impression(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction& OnSuccess = nullptr,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);

#pragma endregion

#pragma region Publication

	/**
	 * Request the Device UUID and get the response.
	 * @param OnSuccess A callback listener that takes a string parameter representing the Device UUID.
	 * @param OnFailure An optional callback listener to be invoked if an error occurs.
	 * <example>
	 * **Example usage**
	 * @code
	 * FAirbridge::FetchDeviceUUID([](const FString& UUID)
	 * {
	 *     // Process UUID
	 * }, [](const FString& ErrorMessage)
	 * {
	 *     // Handle on failure
	 * });
	 * @endcode
	 * </example>
	 */
	static void FetchDeviceUUID(
		const AirbridgeStringParamFunction& OnSuccess,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);

	/**
	 * Request the AirbridgeGeneratedUUID and get the response.
	 * @param OnSuccess A callback listener that takes a string parameter representing the Airbridge Generated UUID.
	 * @param OnFailure An optional callback listener to be invoked if an error occurs.
	 * <example>
	 * **Example usage**
	 * @code
	 * FAirbridge::FetchAirbridgeGeneratedUUID([](const FString& UUID)
	 * {
	 *     // Process UUID
	 * }, [](const FString& ErrorMessage)
	 * {
	 *     // Handle on failure
	 * });
	 * @endcode
	 * </example>
	 */
	static void FetchAirbridgeGeneratedUUID(
		const AirbridgeStringParamFunction& OnSuccess,
		const AirbridgeStringParamFunction& OnFailure = nullptr
	);

	/**
	 * Sets a listener for receiving attribution of install event.
	 * @param OnAttributionReceived A callback listener that receives a `TMap&lt;FString, FString&gt;&amp;` parameter representing the attribution result.
	 * <example>
	 * **Example usage**
	 * @code
	 * FAirbridge::SetOnAttributionReceived([](const TMap<FString, FString>& AttributionResult)
	 * {
	 *     // Process attribution data
	 * });
	 * @endcode
	 * </example>
	 */
	static void SetOnAttributionReceived(const AirbridgeMapParamFunction& OnAttributionReceived);

#pragma endregion 

#pragma region Hybrid

	/**
	 * Creates a script that initialize the web interface to implement web interface manually.
	 * @param WebToken The token to initialize Airbridge Web SDK.
	 * @param PostMessageScript The JavaScript code to post commands from web to app.
	 * (The parameter name used to send messages from web to app is `payload`.)
	 * @return The web interface script.
	 */
	static FString CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript);

	/**
	 * Handles commands from the web interface to implement web interface manually.
	 * @param Command The command to handle.
	 */
	static void HandleWebInterfaceCommand(const FString& Command);

#pragma endregion

	/**
	 * Indicates whether notification was sent by Airbridge to track uninstall of app.
	 * @param Data The notification data to check.
	 * @return `true` if notification was sent by Airbridge, `false` otherwise.
	 */
	static bool IsUninstallTrackingNotification(const TMap<FString, FString>& Data);
};
